@@ -11,10 +11,12 @@ import (
 )
 
 func TestGenerated(t *testing.T) {
+	testenv.NeedsGoPackages(t)
 	// This test fails on 1.18 Kokoro for unknown reasons; in any case, it
 	// suffices to run this test on any builder.
 	testenv.NeedsGo1Point(t, 19)
-	testenv.NeedsGoBuild(t) // This is a lie. We actually need the source code.
+
+	testenv.NeedsLocalXTools(t)
 
 	ok, err := doMain(false)
 	if err != nil {
