@@ -27,10 +27,10 @@ import (
 
 	"golang.org/x/sync/errgroup"
 	"golang.org/x/tools/go/analysis"
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/lsp/filecache"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/internal/bug"
 	"golang.org/x/tools/internal/facts"
 	"golang.org/x/tools/internal/gcimporter"
 	"golang.org/x/tools/internal/memoize"
@@ -154,7 +154,7 @@ import (
 //   Steps:
 //   - define a narrow driver.Snapshot interface with only these methods:
 //        Metadata(PackageID) source.Metadata
-//        GetFile(Context, URI) (source.FileHandle, error)
+//        ReadFile(Context, URI) (source.FileHandle, error)
 //        View() *View // for Options
 //   - define a State type that encapsulates the persistent map
 //     (with its own mutex), and has methods:
@@ -262,8 +262,10 @@ type analyzeSummary struct {
 // TODO(adonovan): generalize and move to a library when we can use generics.
 type actionsMap map[string]*actionSummary
 
-var _ gob.GobEncoder = (actionsMap)(nil)
-var _ gob.GobDecoder = (*actionsMap)(nil)
+var (
+	_ gob.GobEncoder = (actionsMap)(nil)
+	_ gob.GobDecoder = (*actionsMap)(nil)
+)
 
 type actionsMapEntry struct {
 	K string
@@ -370,17 +372,23 @@ func analyzeImpl(ctx context.Context, snapshot *snapshot, analyzers []*analysis.
 		return nil, fmt.Errorf("no metadata for %s", id)
 	}
 
+	// Also, load the contents of each "compiled" Go file through
+	// the snapshot's cache.
+	// (These are all cache hits as files are pre-loaded following packages.Load)
+	compiledGoFiles := make([]source.FileHandle, len(m.CompiledGoFiles))
+	for i, uri := range m.CompiledGoFiles {
+		fh, err := snapshot.ReadFile(ctx, uri)
+		if err != nil {
+			return nil, err // e.g. canceled
+		}
+		compiledGoFiles[i] = fh
+	}
+
 	// Recursively analyze each "vertical" dependency
 	// for its types.Package and (perhaps) analysis.Facts.
 	// If any of them fails to produce a package, we cannot continue.
 	// We request only the analyzers that produce facts.
-	//
-	// Also, load the contents of each "compiled" Go file through
-	// the snapshot's cache.
-	//
-	// Both loops occur in parallel, and parallel with each other.
 	vdeps := make(map[PackageID]*analyzeSummary)
-	compiledGoFiles := make([]source.FileHandle, len(m.CompiledGoFiles))
 	{
 		var group errgroup.Group
 
@@ -408,19 +416,6 @@ func analyzeImpl(ctx context.Context, snapshot *snapshot, analyzers []*analysis.
 			})
 		}
 
-		// Read file contents.
-		// (In practice these will be cache hits
-		// on reads done by the initial workspace load
-		// or after a change modification event.)
-		for i, uri := range m.CompiledGoFiles {
-			i, uri := i, uri
-			group.Go(func() error {
-				fh, err := snapshot.GetFile(ctx, uri) // ~25us
-				compiledGoFiles[i] = fh
-				return err // e.g. cancelled
-			})
-		}
-
 		if err := group.Wait(); err != nil {
 			return nil, err
 		}
@@ -450,10 +445,8 @@ func analyzeImpl(ctx context.Context, snapshot *snapshot, analyzers []*analysis.
 	if data, err := filecache.Get(cacheKind, key); err == nil {
 		// cache hit
 		mustDecode(data, &summary)
-
 	} else if err != filecache.ErrNotFound {
 		return nil, bug.Errorf("internal error reading shared cache: %v", err)
-
 	} else {
 		// Cache miss: do the work.
 		var err error
@@ -573,7 +566,6 @@ func analysisCacheKey(analyzers []*analysis.Analyzer, m *source.Metadata, compil
 // actuallyAnalyze implements the cache-miss case.
 // This function does not access the snapshot.
 func actuallyAnalyze(ctx context.Context, analyzers []*analysis.Analyzer, m *source.Metadata, vdeps map[PackageID]*analyzeSummary, compiledGoFiles []source.FileHandle) (*analyzeSummary, error) {
-
 	// Create a local FileSet for processing this package only.
 	fset := token.NewFileSet()
 
@@ -587,7 +579,9 @@ func actuallyAnalyze(ctx context.Context, analyzers []*analysis.Analyzer, m *sou
 			group.Go(func() error {
 				// Call parseGoImpl directly, not the caching wrapper,
 				// as cached ASTs require the global FileSet.
-				pgf, err := parseGoImpl(ctx, fset, fh, source.ParseFull)
+				// ast.Object resolution is unfortunately an implied part of the
+				// go/analysis contract.
+				pgf, err := parseGoImpl(ctx, fset, fh, source.ParseFull&^source.SkipObjectResolution)
 				parsed[i] = pgf
 				return err
 			})
@@ -748,7 +742,7 @@ func typeCheckForAnalysis(fset *token.FileSet, parsed []*source.ParsedGoFile, m
 			return nil, bug.Errorf("missing export data for %q", pkgPath)
 		}
 		hashExport(pkgPath, export)
-		imported, err := gcimporter.IImportShallow(fset, importMap, export, string(pkgPath), insert)
+		imported, err := gcimporter.IImportShallow(fset, gcimporter.GetPackageFromMap(importMap), export, string(pkgPath), insert)
 		if err != nil {
 			return nil, bug.Errorf("invalid export data for %q: %v", pkgPath, err)
 		}
@@ -957,7 +951,7 @@ func (act *action) exec() (interface{}, *actionSummary, error) {
 	}
 
 	// Gather analysis Result values from horizontal dependencies.
-	var inputs = make(map[*analysis.Analyzer]interface{})
+	inputs := make(map[*analysis.Analyzer]interface{})
 	for _, dep := range act.hdeps {
 		inputs[dep.a] = dep.result
 	}
@@ -1150,12 +1144,28 @@ func mustEncode(x interface{}) []byte {
 	return buf.Bytes()
 }
 
+// TODO(rfindley): based on profiling, we should consider using JSON encoding
+// throughout, rather than gob encoding.
+func mustJSONEncode(x interface{}) []byte {
+	data, err := json.Marshal(x)
+	if err != nil {
+		log.Fatalf("internal error marshalling %T: %v", data, err)
+	}
+	return data
+}
+
 func mustDecode(data []byte, ptr interface{}) {
 	if err := gob.NewDecoder(bytes.NewReader(data)).Decode(ptr); err != nil {
 		log.Fatalf("internal error decoding %T: %v", ptr, err)
 	}
 }
 
+func mustJSONDecode(data []byte, ptr interface{}) {
+	if err := json.Unmarshal(data, ptr); err != nil {
+		log.Fatalf("internal error unmarshalling %T: %v", ptr, err)
+	}
+}
+
 // -- data types for serialization of analysis.Diagnostic and source.Diagnostic --
 
 type gobDiagnostic struct {
@@ -1237,7 +1247,7 @@ func toGobDiagnostic(posToLocation func(start, end token.Pos) (protocol.Location
 		// Severity for analysis diagnostics is dynamic, based on user
 		// configuration per analyzer.
 		// Code and CodeHref are unset for Analysis diagnostics,
-		// TODO(rfindley): set Code fields if/when golang/go#57906 is accepted.
+		// TODO(rfindley): derive Code fields from diag.URL.
 		Source:         diag.Category,
 		Message:        diag.Message,
 		SuggestedFixes: fixes,
