@@ -8,7 +8,6 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"errors"
 	"flag"
 	"fmt"
 	"go/token"
@@ -120,6 +119,14 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //
 // The following markers are supported within marker tests:
 //
+//   - codeaction(kind, start, end, golden): specifies a codeaction to request
+//     for the given range. To support multi-line ranges, the range is defined
+//     to be between start.Start and end.End. The golden directory contains
+//     changed file content after the code action is applied.
+//
+//   - codeactionerr(kind, start, end, wantError): specifies a codeaction that
+//     fails with an error that matches the expectation.
+//
 //   - complete(location, ...labels): specifies expected completion results at
 //     the given location.
 //
@@ -128,7 +135,13 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     a 1:1 correspondence between observed diagnostics and diag annotations
 //
 //   - def(src, dst location): perform a textDocument/definition request at
-//     the src location, and check the the result points to the dst location.
+//     the src location, and check the result points to the dst location.
+//
+//   - format(golden): perform a textDocument/format request for the enclosing
+//     file, and compare against the named golden file. If the formatting
+//     request succeeds, the golden file must contain the resulting formatted
+//     source. If the formatting request fails, the golden file must contain
+//     the error message.
 //
 //   - hover(src, dst location, g Golden): perform a textDocument/hover at the
 //     src location, and checks that the result is the dst location, with hover
@@ -170,6 +183,14 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     location information. There is no point to using more than one
 //     @symbol marker in a given file.
 //
+//   - workspacesymbol(query, golden): makes a workspace/symbol request for the
+//     given query, formats the response with one symbol per line, and compares
+//     against the named golden file. As workspace symbols are by definition a
+//     workspace-wide request, the location of the workspace symbol marker does
+//     not matter. Each line is of the form:
+//
+//     location name kind
+//
 // # Argument conversion
 //
 // Marker arguments are first parsed by the go/expect package, which accepts
@@ -417,6 +438,11 @@ type marker struct {
 	note *expect.Note
 }
 
+// server returns the LSP server for the marker test run.
+func (m marker) server() protocol.Server {
+	return m.run.env.Editor.Server
+}
+
 // errorf reports an error with a prefix indicating the position of the marker note.
 //
 // It formats the error message using mark.sprintf.
@@ -482,17 +508,21 @@ arity:
 // Marker funcs should not mutate the test environment (e.g. via opening files
 // or applying edits in the editor).
 var markerFuncs = map[string]markerFunc{
-	"complete":       makeMarkerFunc(completeMarker),
-	"def":            makeMarkerFunc(defMarker),
-	"diag":           makeMarkerFunc(diagMarker),
-	"hover":          makeMarkerFunc(hoverMarker),
-	"implementation": makeMarkerFunc(implementationMarker),
-	"loc":            makeMarkerFunc(locMarker),
-	"rename":         makeMarkerFunc(renameMarker),
-	"renameerr":      makeMarkerFunc(renameErrMarker),
-	"suggestedfix":   makeMarkerFunc(suggestedfixMarker),
-	"symbol":         makeMarkerFunc(symbolMarker),
-	"refs":           makeMarkerFunc(refsMarker),
+	"codeaction":      makeMarkerFunc(codeActionMarker),
+	"codeactionerr":   makeMarkerFunc(codeActionErrMarker),
+	"complete":        makeMarkerFunc(completeMarker),
+	"def":             makeMarkerFunc(defMarker),
+	"diag":            makeMarkerFunc(diagMarker),
+	"hover":           makeMarkerFunc(hoverMarker),
+	"format":          makeMarkerFunc(formatMarker),
+	"implementation":  makeMarkerFunc(implementationMarker),
+	"loc":             makeMarkerFunc(locMarker),
+	"rename":          makeMarkerFunc(renameMarker),
+	"renameerr":       makeMarkerFunc(renameErrMarker),
+	"suggestedfix":    makeMarkerFunc(suggestedfixMarker),
+	"symbol":          makeMarkerFunc(symbolMarker),
+	"refs":            makeMarkerFunc(refsMarker),
+	"workspacesymbol": makeMarkerFunc(workspaceSymbolMarker),
 }
 
 // markerTest holds all the test data extracted from a test txtar archive.
@@ -852,8 +882,21 @@ func (run *markerTestRun) fmtPos(pos token.Pos) string {
 // archive-relative paths for files and including the line number in the full
 // archive file.
 func (run *markerTestRun) fmtLoc(loc protocol.Location) string {
+	formatted := run.fmtLocDetails(loc, true)
+	if formatted == "" {
+		run.env.T.Errorf("unable to find %s in test archive", loc)
+		return "<invalid location>"
+	}
+	return formatted
+}
+
+// See fmtLoc. If includeTxtPos is not set, the position in the full archive
+// file is omitted.
+//
+// If the location cannot be found within the archive, fmtLocDetails returns "".
+func (run *markerTestRun) fmtLocDetails(loc protocol.Location, includeTxtPos bool) string {
 	if loc == (protocol.Location{}) {
-		return "<missing location>"
+		return ""
 	}
 	lines := bytes.Count(run.test.archive.Comment, []byte("\n"))
 	var name string
@@ -867,8 +910,7 @@ func (run *markerTestRun) fmtLoc(loc protocol.Location) string {
 		lines += bytes.Count(f.Data, []byte("\n"))
 	}
 	if name == "" {
-		run.env.T.Errorf("unable to find %s in test archive", loc)
-		return "<invalid location>"
+		return ""
 	}
 	m, err := run.env.Editor.Mapper(name)
 	if err != nil {
@@ -893,7 +935,11 @@ func (run *markerTestRun) fmtLoc(loc protocol.Location) string {
 		}
 	}
 
-	return fmt.Sprintf("%s:%s (%s:%s)", name, innerSpan, run.test.name, outerSpan)
+	if includeTxtPos {
+		return fmt.Sprintf("%s:%s (%s:%s)", name, innerSpan, run.test.name, outerSpan)
+	} else {
+		return fmt.Sprintf("%s:%s", name, innerSpan)
+	}
 }
 
 // makeMarkerFunc uses reflection to create a markerFunc for the given func value.
@@ -1173,6 +1219,40 @@ func defMarker(mark marker, src, dst protocol.Location) {
 	}
 }
 
+// formatMarker implements the @format marker.
+func formatMarker(mark marker, golden *Golden) {
+	edits, err := mark.server().Formatting(mark.run.env.Ctx, &protocol.DocumentFormattingParams{
+		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
+	})
+	var got []byte
+	if err != nil {
+		got = []byte(err.Error() + "\n") // all golden content is newline terminated
+	} else {
+		env := mark.run.env
+		filename := env.Sandbox.Workdir.URIToPath(mark.uri())
+		mapper, err := env.Editor.Mapper(filename)
+		if err != nil {
+			mark.errorf("Editor.Mapper(%s) failed: %v", filename, err)
+		}
+
+		got, _, err = source.ApplyProtocolEdits(mapper, edits)
+		if err != nil {
+			mark.errorf("ApplyProtocolEdits failed: %v", err)
+			return
+		}
+	}
+
+	want, ok := golden.Get(mark.run.env.T, "", got)
+	if !ok {
+		mark.errorf("missing golden file @%s", golden.id)
+		return
+	}
+
+	if diff := compare.Bytes(want, got); diff != "" {
+		mark.errorf("golden file @%s does not match format results:\n%s", golden.id, diff)
+	}
+}
+
 // hoverMarker implements the @hover marker, running textDocument/hover at the
 // given src location and asserting that the resulting hover is over the dst
 // location (typically a span surrounding src), and that the markdown content
@@ -1215,20 +1295,20 @@ func locMarker(mark marker, name expect.Identifier, loc protocol.Location) {
 // diagMarker implements the @diag marker. It eliminates diagnostics from
 // the observed set in mark.test.
 func diagMarker(mark marker, loc protocol.Location, re *regexp.Regexp) {
-	if _, err := removeDiagnostic(mark, loc, re); err != nil {
-		mark.errorf("%v", err)
+	if _, ok := removeDiagnostic(mark, loc, re); !ok {
+		mark.errorf("no diagnostic at %v matches %q", loc, re)
 	}
 }
 
-func removeDiagnostic(mark marker, loc protocol.Location, re *regexp.Regexp) (protocol.Diagnostic, error) {
+func removeDiagnostic(mark marker, loc protocol.Location, re *regexp.Regexp) (protocol.Diagnostic, bool) {
 	diags := mark.run.diags[loc]
 	for i, diag := range diags {
 		if re.MatchString(diag.Message) {
 			mark.run.diags[loc] = append(diags[:i], diags[i+1:]...)
-			return diag, nil
+			return diag, true
 		}
 	}
-	return protocol.Diagnostic{}, errors.New(mark.sprintf("no diagnostic at %v matches %q", loc, re))
+	return protocol.Diagnostic{}, false
 }
 
 // renameMarker implements the @rename(location, new, golden) marker.
@@ -1265,57 +1345,60 @@ func rename(env *Env, loc protocol.Location, newName string) (map[string][]byte,
 		return nil, err
 	}
 
-	return applyDocumentChanges(env, editMap.DocumentChanges)
+	fileChanges := make(map[string][]byte)
+	if err := applyDocumentChanges(env, editMap.DocumentChanges, fileChanges); err != nil {
+		return nil, fmt.Errorf("applying document changes: %v", err)
+	}
+	return fileChanges, nil
 }
 
-// applyDocumentChanges returns the effect of applying the document
-// changes to the contents of the Editor buffers. The actual editor
-// buffers are unchanged.
-func applyDocumentChanges(env *Env, changes []protocol.DocumentChanges) (map[string][]byte, error) {
-	result := make(map[string][]byte)
+// applyDocumentChanges applies the given document changes to the editor buffer
+// content, recording the resulting contents in the fileChanges map. It is an
+// error for a change to an edit a file that is already present in the
+// fileChanges map.
+func applyDocumentChanges(env *Env, changes []protocol.DocumentChanges, fileChanges map[string][]byte) error {
+	getMapper := func(path string) (*protocol.Mapper, error) {
+		if _, ok := fileChanges[path]; ok {
+			return nil, fmt.Errorf("internal error: %s is already edited", path)
+		}
+		return env.Editor.Mapper(path)
+	}
+
 	for _, change := range changes {
 		if change.RenameFile != nil {
 			// rename
 			oldFile := env.Sandbox.Workdir.URIToPath(change.RenameFile.OldURI)
-			newFile := env.Sandbox.Workdir.URIToPath(change.RenameFile.NewURI)
-			mapper, err := env.Editor.Mapper(oldFile)
+			mapper, err := getMapper(oldFile)
 			if err != nil {
-				return nil, err
+				return err
 			}
-			result[newFile] = mapper.Content
-
+			newFile := env.Sandbox.Workdir.URIToPath(change.RenameFile.NewURI)
+			fileChanges[newFile] = mapper.Content
 		} else {
 			// edit
 			filename := env.Sandbox.Workdir.URIToPath(change.TextDocumentEdit.TextDocument.URI)
-			mapper, err := env.Editor.Mapper(filename)
+			mapper, err := getMapper(filename)
 			if err != nil {
-				return nil, err
+				return err
 			}
 			patched, _, err := source.ApplyProtocolEdits(mapper, change.TextDocumentEdit.Edits)
 			if err != nil {
-				return nil, err
+				return err
 			}
-			result[filename] = patched
+			fileChanges[filename] = patched
 		}
 	}
 
-	return result, nil
+	return nil
 }
 
-// suggestedfixMarker implements the @suggestedfix(location, regexp,
-// kind, golden) marker. It acts like @diag(location, regexp), to set
-// the expectation of a diagnostic, but then it applies the first code
-// action of the specified kind suggested by the matched diagnostic.
-func suggestedfixMarker(mark marker, loc protocol.Location, re *regexp.Regexp, actionKind string, golden *Golden) {
-	// Find and remove the matching diagnostic.
-	diag, err := removeDiagnostic(mark, loc, re)
-	if err != nil {
-		mark.errorf("%v", err)
-		return
-	}
+func codeActionMarker(mark marker, actionKind string, start, end protocol.Location, golden *Golden) {
+	// Request the range from start.Start to end.End.
+	loc := start
+	loc.Range.End = end.Range.End
 
 	// Apply the fix it suggests.
-	changed, err := suggestedfix(mark.run.env, loc, diag, actionKind)
+	changed, err := codeAction(mark.run.env, loc.URI, loc.Range, actionKind, nil)
 	if err != nil {
 		mark.errorf("suggestedfix failed: %v. (Use @suggestedfixerr for expected errors.)", err)
 		return
@@ -1325,19 +1408,59 @@ func suggestedfixMarker(mark marker, loc protocol.Location, re *regexp.Regexp, a
 	checkChangedFiles(mark, changed, golden)
 }
 
-func suggestedfix(env *Env, loc protocol.Location, diag protocol.Diagnostic, actionKind string) (map[string][]byte, error) {
+func codeActionErrMarker(mark marker, actionKind string, start, end protocol.Location, wantErr wantError) {
+	loc := start
+	loc.Range.End = end.Range.End
+	_, err := codeAction(mark.run.env, loc.URI, loc.Range, actionKind, nil)
+	wantErr.check(mark, err)
+}
 
+// suggestedfixMarker implements the @suggestedfix(location, regexp,
+// kind, golden) marker. It acts like @diag(location, regexp), to set
+// the expectation of a diagnostic, but then it applies the first code
+// action of the specified kind suggested by the matched diagnostic.
+func suggestedfixMarker(mark marker, loc protocol.Location, re *regexp.Regexp, actionKind string, golden *Golden) {
+	// Find and remove the matching diagnostic.
+	diag, ok := removeDiagnostic(mark, loc, re)
+	if !ok {
+		mark.errorf("no diagnostic at %v matches %q", loc, re)
+		return
+	}
+
+	// Apply the fix it suggests.
+	changed, err := codeAction(mark.run.env, loc.URI, diag.Range, actionKind, &diag)
+	if err != nil {
+		mark.errorf("suggestedfix failed: %v. (Use @suggestedfixerr for expected errors.)", err)
+		return
+	}
+
+	// Check the file state.
+	checkChangedFiles(mark, changed, golden)
+}
+
+// codeAction executes a textDocument/codeAction request for the specified
+// location and kind. If diag is non-nil, it is used as the code action
+// context.
+//
+// The resulting map contains resulting file contents after the code action is
+// applied. Currently, this function does not support code actions that return
+// edits directly; it only supports code action commands.
+func codeAction(env *Env, uri protocol.DocumentURI, rng protocol.Range, actionKind string, diag *protocol.Diagnostic) (map[string][]byte, error) {
 	// Request all code actions that apply to the diagnostic.
 	// (The protocol supports filtering using Context.Only={actionKind}
 	// but we can give a better error if we don't filter.)
-	actions, err := env.Editor.Server.CodeAction(env.Ctx, &protocol.CodeActionParams{
-		TextDocument: protocol.TextDocumentIdentifier{URI: loc.URI},
-		Range:        diag.Range,
+	params := &protocol.CodeActionParams{
+		TextDocument: protocol.TextDocumentIdentifier{URI: uri},
+		Range:        rng,
 		Context: protocol.CodeActionContext{
-			Only:        nil, // => all kinds
-			Diagnostics: []protocol.Diagnostic{diag},
+			Only: nil, // => all kinds
 		},
-	})
+	}
+	if diag != nil {
+		params.Context.Diagnostics = []protocol.Diagnostic{*diag}
+	}
+
+	actions, err := env.Editor.Server.CodeAction(env.Ctx, params)
 	if err != nil {
 		return nil, err
 	}
@@ -1357,40 +1480,56 @@ func suggestedfix(env *Env, loc protocol.Location, diag protocol.Diagnostic, act
 	}
 	action := candidates[0]
 
+	// Apply the codeAction.
+	//
+	// Spec:
+	//  "If a code action provides an edit and a command, first the edit is
+	//  executed and then the command."
+	fileChanges := make(map[string][]byte)
 	// An action may specify an edit and/or a command, to be
 	// applied in that order. But since applyDocumentChanges(env,
 	// action.Edit.DocumentChanges) doesn't compose, for now we
 	// assert that all commands used in the @suggestedfix tests
 	// return only a command.
-	if action.Edit.DocumentChanges != nil {
-		env.T.Errorf("internal error: discarding unexpected CodeAction{Kind=%s, Title=%q}.Edit.DocumentChanges", action.Kind, action.Title)
-	}
-	if action.Command == nil {
-		return nil, fmt.Errorf("missing CodeAction{Kind=%s, Title=%q}.Command", action.Kind, action.Title)
+	if action.Edit != nil {
+		if action.Edit.Changes != nil {
+			env.T.Errorf("internal error: discarding unexpected CodeAction{Kind=%s, Title=%q}.Edit.Changes", action.Kind, action.Title)
+		}
+		if action.Edit.DocumentChanges != nil {
+			if err := applyDocumentChanges(env, action.Edit.DocumentChanges, fileChanges); err != nil {
+				return nil, fmt.Errorf("applying document changes: %v", err)
+			}
+		}
 	}
 
-	// This is a typical CodeAction command:
-	//
-	//   Title:     "Implement error"
-	//   Command:   gopls.apply_fix
-	//   Arguments: [{"Fix":"stub_methods","URI":".../a.go","Range":...}}]
-	//
-	// The client makes an ExecuteCommand RPC to the server,
-	// which dispatches it to the ApplyFix handler.
-	// ApplyFix dispatches to the "stub_methods" suggestedfix hook (the meat).
-	// The server then makes an ApplyEdit RPC to the client,
-	// whose Awaiter hook gathers the edits instead of applying them.
+	if action.Command != nil {
+		// This is a typical CodeAction command:
+		//
+		//   Title:     "Implement error"
+		//   Command:   gopls.apply_fix
+		//   Arguments: [{"Fix":"stub_methods","URI":".../a.go","Range":...}}]
+		//
+		// The client makes an ExecuteCommand RPC to the server,
+		// which dispatches it to the ApplyFix handler.
+		// ApplyFix dispatches to the "stub_methods" suggestedfix hook (the meat).
+		// The server then makes an ApplyEdit RPC to the client,
+		// whose Awaiter hook gathers the edits instead of applying them.
 
-	_ = env.Awaiter.takeDocumentChanges() // reset (assuming Env is confined to this thread)
+		_ = env.Awaiter.takeDocumentChanges() // reset (assuming Env is confined to this thread)
 
-	if _, err := env.Editor.Server.ExecuteCommand(env.Ctx, &protocol.ExecuteCommandParams{
-		Command:   action.Command.Command,
-		Arguments: action.Command.Arguments,
-	}); err != nil {
-		env.T.Fatalf("error converting command %q to edits: %v", action.Command.Command, err)
+		if _, err := env.Editor.Server.ExecuteCommand(env.Ctx, &protocol.ExecuteCommandParams{
+			Command:   action.Command.Command,
+			Arguments: action.Command.Arguments,
+		}); err != nil {
+			env.T.Fatalf("error converting command %q to edits: %v", action.Command.Command, err)
+		}
+
+		if err := applyDocumentChanges(env, env.Awaiter.takeDocumentChanges(), fileChanges); err != nil {
+			return nil, fmt.Errorf("applying document changes from command: %v", err)
+		}
 	}
 
-	return applyDocumentChanges(env, env.Awaiter.takeDocumentChanges())
+	return fileChanges, nil
 }
 
 // TODO(adonovan): suggestedfixerr
@@ -1398,7 +1537,7 @@ func suggestedfix(env *Env, loc protocol.Location, diag protocol.Diagnostic, act
 // refsMarker implements the @refs marker.
 func refsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
 	refs := func(includeDeclaration bool, want []protocol.Location) error {
-		got, err := mark.run.env.Editor.Server.References(mark.run.env.Ctx, &protocol.ReferenceParams{
+		got, err := mark.server().References(mark.run.env.Ctx, &protocol.ReferenceParams{
 			TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
 			Context: protocol.ReferenceContext{
 				IncludeDeclaration: includeDeclaration,
@@ -1428,7 +1567,7 @@ func refsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
 
 // implementationMarker implements the @implementation marker.
 func implementationMarker(mark marker, src protocol.Location, want ...protocol.Location) {
-	got, err := mark.run.env.Editor.Server.Implementation(mark.run.env.Ctx, &protocol.ImplementationParams{
+	got, err := mark.server().Implementation(mark.run.env.Ctx, &protocol.ImplementationParams{
 		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
 	})
 	if err != nil {
@@ -1443,7 +1582,7 @@ func implementationMarker(mark marker, src protocol.Location, want ...protocol.L
 // symbolMarker implements the @symbol marker.
 func symbolMarker(mark marker, golden *Golden) {
 	// Retrieve information about all symbols in this file.
-	symbols, err := mark.run.env.Editor.Server.DocumentSymbol(mark.run.env.Ctx, &protocol.DocumentSymbolParams{
+	symbols, err := mark.server().DocumentSymbol(mark.run.env.Ctx, &protocol.DocumentSymbolParams{
 		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
 	})
 	if err != nil {
@@ -1526,3 +1665,37 @@ func compareLocations(mark marker, got, want []protocol.Location) error {
 	}
 	return nil
 }
+
+func workspaceSymbolMarker(mark marker, query string, golden *Golden) {
+	params := &protocol.WorkspaceSymbolParams{
+		Query: query,
+	}
+
+	gotSymbols, err := mark.server().Symbol(mark.run.env.Ctx, params)
+	if err != nil {
+		mark.errorf("Symbol(%q) failed: %v", query, err)
+		return
+	}
+	var got bytes.Buffer
+	for _, s := range gotSymbols {
+		// Omit the txtar position of the symbol location; otherwise edits to the
+		// txtar archive lead to unexpected failures.
+		loc := mark.run.fmtLocDetails(s.Location, false)
+		// TODO(rfindley): can we do better here, by detecting if the location is
+		// relative to GOROOT?
+		if loc == "" {
+			loc = "<unknown>"
+		}
+		fmt.Fprintf(&got, "%s %s %s\n", loc, s.Name, s.Kind)
+	}
+
+	want, ok := golden.Get(mark.run.env.T, "", got.Bytes())
+	if !ok {
+		mark.errorf("missing golden file @%s", golden.id)
+		return
+	}
+
+	if diff := compare.Bytes(want, got.Bytes()); diff != "" {
+		mark.errorf("Symbol(%q) mismatch:\n%s", query, diff)
+	}
+}
