@@ -10,9 +10,9 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/hooks"
 	. "golang.org/x/tools/gopls/internal/lsp/regtest"
-	"golang.org/x/tools/internal/bug"
 	"golang.org/x/tools/internal/testenv"
 
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
@@ -232,11 +232,9 @@ package ma
 	})
 }
 
+// TODO(rfindley): audit/clean up call sites for this helper, to ensure
+// consistent test errors.
 func compareCompletionLabels(want []string, gotItems []protocol.CompletionItem) string {
-	if len(gotItems) != len(want) {
-		return fmt.Sprintf("got %v completion(s), want %v", len(gotItems), len(want))
-	}
-
 	var got []string
 	for _, item := range gotItems {
 		got = append(got, item.Label)
@@ -246,12 +244,13 @@ func compareCompletionLabels(want []string, gotItems []protocol.CompletionItem)
 		}
 	}
 
-	for i, v := range got {
-		if v != want[i] {
-			return fmt.Sprintf("%d completion result not the same: got %q, want %q", i, v, want[i])
-		}
+	if len(got) == 0 && len(want) == 0 {
+		return "" // treat nil and the empty slice as equivalent
 	}
 
+	if diff := cmp.Diff(want, got); diff != "" {
+		return fmt.Sprintf("completion item mismatch (-want +got):\n%s", diff)
+	}
 	return ""
 }
 
@@ -298,7 +297,7 @@ func _() {
 		if len(completions.Items) == 0 {
 			t.Fatalf("no completion items")
 		}
-		env.AcceptCompletion(loc, completions.Items[0])
+		env.AcceptCompletion(loc, completions.Items[0]) // adds blah import to main.go
 		env.Await(env.DoneWithChange())
 
 		// Trigger completions once again for the blah.<> selector.
@@ -517,7 +516,7 @@ func main() {
 	WithOptions(
 		WindowsLineEndings(),
 	).Run(t, src, func(t *testing.T, env *Env) {
-		// Trigger unimported completions for the example.com/blah package.
+		// Trigger unimported completions for the mod.com package.
 		env.OpenFile("main.go")
 		env.Await(env.DoneWithOpen())
 		loc := env.RegexpSearch("main.go", "Sqr()")
@@ -535,9 +534,50 @@ func main() {
 	})
 }
 
+func TestPackageMemberCompletionAfterSyntaxError(t *testing.T) {
+	// This test documents the current broken behavior due to golang/go#58833.
+	const src = `
+-- go.mod --
+module mod.com
+
+go 1.14
+
+-- main.go --
+package main
+
+import "math"
+
+func main() {
+	math.Sqrt(,0)
+	math.Ldex
+}
+`
+	Run(t, src, func(t *testing.T, env *Env) {
+		env.OpenFile("main.go")
+		env.Await(env.DoneWithOpen())
+		loc := env.RegexpSearch("main.go", "Ldex()")
+		completions := env.Completion(loc)
+		if len(completions.Items) == 0 {
+			t.Fatalf("no completion items")
+		}
+		env.AcceptCompletion(loc, completions.Items[0])
+		env.Await(env.DoneWithChange())
+		got := env.BufferText("main.go")
+		// The completion of math.Ldex after the syntax error on the
+		// previous line is not "math.Ldexp" but "math.Ldexmath.Abs".
+		// (In VSCode, "Abs" wrongly appears in the completion menu.)
+		// This is a consequence of poor error recovery in the parser
+		// causing "math.Ldex" to become a BadExpr.
+		want := "package main\n\nimport \"math\"\n\nfunc main() {\n\tmath.Sqrt(,0)\n\tmath.Ldexmath.Abs(${1:})\n}\n"
+		if diff := cmp.Diff(want, got); diff != "" {
+			t.Errorf("unimported completion (-want +got):\n%s", diff)
+		}
+	})
+}
+
 func TestDefinition(t *testing.T) {
 	testenv.NeedsGo1Point(t, 17) // in go1.16, The FieldList in func x is not empty
-	stuff := `
+	files := `
 -- go.mod --
 module mod.com
 
@@ -569,20 +609,16 @@ package foo
 		{"func Te(t *testing.T)", "Te", []string{"TestMain", "Test"}},
 	}
 	fname := "a_test.go"
-	Run(t, stuff, func(t *testing.T, env *Env) {
+	Run(t, files, func(t *testing.T, env *Env) {
 		env.OpenFile(fname)
 		env.Await(env.DoneWithOpen())
-		for _, tst := range tests {
-			env.SetBufferContent(fname, "package foo\n"+tst.line)
-			loc := env.RegexpSearch(fname, tst.pat)
-			loc.Range.Start.Character += uint32(protocol.UTF16Len([]byte(tst.pat)))
+		for _, test := range tests {
+			env.SetBufferContent(fname, "package foo\n"+test.line)
+			loc := env.RegexpSearch(fname, test.pat)
+			loc.Range.Start.Character += uint32(protocol.UTF16Len([]byte(test.pat)))
 			completions := env.Completion(loc)
-			result := compareCompletionLabels(tst.want, completions.Items)
-			if result != "" {
-				t.Errorf("\npat:%q line:%q failed: %s:%q", tst.pat, tst.line, result, tst.want)
-				for i, it := range completions.Items {
-					t.Errorf("%d got %q %q", i, it.Label, it.Detail)
-				}
+			if diff := compareCompletionLabels(test.want, completions.Items); diff != "" {
+				t.Error(diff)
 			}
 		}
 	})
